# figurenGruppe.py

from figuren.grafikfenster import Zeichenflaeche
from figuren.figur import Figur

### ---------------------------------------------------
class FigurenGruppe(Figur):
    """Klasse FigurenGruppe ermoeglicht das Zeichnen
    und Bearbeiten von FigurenGruppen.
    Die Kindobjekte werden mit relativer Position eingebaut."""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 winkel=0):
        self.__figuren = []
        Figur.__init__(self, xPos, yPos, 1, 1, winkel, 'black')
        ## die Figurengruppe hat keine eigene Sichtbarkeit
        
    def AendereAbmessungen(self, neueBreite, neueHoehe):
        """Veraendernde Methode fuer die Breite und Hoehe
        Die relativen Positionen werden korrigiert."""
        faktorX,faktorY = 1.0,1.0
        if self.GibBreite()>0:
            faktorX = float(neueBreite)/self.GibBreite()
        if self.GibHoehe()>0:
            faktorY = float(neueHoehe)/self.GibHoehe()
        self._Figur__breite = neueBreite
        self._Figur__hoehe = neueHoehe
        for figur in self.__figuren:
            b = faktorX * figur.GibBreite()
            h = faktorY * figur.GibHoehe()
            figur.AendereAbmessungen(b, h)
            dx = figur.GibXPosition() * (faktorX - 1.0)
            dy = figur.GibYPosition() * (faktorY - 1.0)
            figur.Bewege(dx, dy)
        
    def AendereFarbe(self, neueFarbe='black'):
        """Gruppen sollten ggf. keine eigene Farbe haben"""
        for figur in self.__figuren:
            figur.AendereFarbe(neueFarbe)
        
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Gruppen sollten ggf. keine eigene Fuellfarbe haben"""
        print('Guppen sollten ggf. keine eigene Fuellfarbe haben')
        for figur in self.__figuren:
            figur.AendereFuellfarbe(neueFuellfarbe)
        
    def GibFigur(self):
        """Definiert den Pfad."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        return path

    def __AktualisiereAbmessungen(self):
        """Interne Methode zum Aktualisieren der Abmessungen
        nach Einbau oder Entfernen einer neuen Figur"""
        b,h = 1,1
        for figur in self.__figuren:
            b=max(figur.GibXPosition()+figur.GibBreite(), b)
            h=max(figur.GibYPosition()+figur.GibHoehe(), h)
        self._Figur__breite,self._Figur__hoehe=b,h

    def FuegeHinzu(self, figur):
        """fuegt der Gruppe eine Figur hinzu
        Voraussetzung fuer richtige Werte ist winkel=0 bei Kindern"""
        self.Verberge()
        if figur not in self.__figuren:
            self.__figuren.append(figur)
            figur.SetzeGruppe(self)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def Entferne(self, figur):
        """entfernt eine Figur aus der Gruppe"""
        self.Verberge()
        if figur in self.__figuren:
            self.__figuren.remove(figur)
            figur.SetzeGruppe(None)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def AendereReihenfolge(self, von, nach):
        """aendert die Reihenfolge, in der die Figuren
        der Gruppe dargestellt werden"""
        anz=len(self.__figuren)
        if (von<1) or (von>anz) or (nach<1) or (nach>anz) or (von==nach): return
        self.Verberge()
        tausch=self.__figuren[von-1]
        self.__figuren[von-1]=self.__figuren[nach-1]
        self.__figuren[nach-1]=tausch
        self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        for figur in self.__figuren:
            figur.Verberge()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        for figur in self.__figuren:
            figur.Zeige()
